from django.conf import settings
from django.contrib.auth.models import Group
import logging
from django.db.models.fields import UUIDField
from urnparse import URN8141
from django.db.models import (
    Model,
    ForeignKey,
    CASCADE,
)


class PerunGroup(Model):
    group = ForeignKey('Group', editable=False, on_delete=CASCADE)
    uuid = UUIDField(editable=False)


logger = logging.getLogger(__name__)


def user_groups(strategy, details, user=None, *args, **kwargs):
    """
        Update user group membership using data from OpenId provider.

        :raises ValueError
    """
    print(f"{kwargs}")
    group_attr = kwargs['response'].get(settings.REMOTE_GROUP_KEY, None)
    if not group_attr:
        return
    internal_group_attr = [URN8141.from_string(g).specific_string.decoded for g in settings.REMOTE_GROUP_FILTER(group_attr)]
    # Create valid nonexist groups
    for g in internal_group_attr:
        try:
            group = Group.objects.create(name=g)
            # PerunGroup.objects.create(group=group, uuid=)
        except:
            pass
    if user:
        # Remove user from any extra groups that weren't provided by OpenId
        remote_groups = Group.objects.filter(name__in=internal_group_attr)
        extra_groups = user.groups.exclude(name__in=[g.name for g in remote_groups] + settings.PROTECTED_GROUPS)
        # if user is superuser means may have groups assigned for test
        if not user.is_superuser:
            for eg in extra_groups:
                logger.debug('Removing {} from group {}'.format(user.username, eg.name))
                eg.user_set.remove(user)

        # Add user to groups provided by OpenId
        local_groups = user.groups.all()
        new_memberships = remote_groups.exclude(name__in=[g.name for g in local_groups] + settings.PROTECTED_GROUPS)
        for ng in new_memberships:
            logger.debug('Adding {} to group {}'.format(user.username, ng.name))
            user.groups.add(ng.pk)



from django.contrib.auth import get_user_model
from urllib.parse import urlencode

from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist
from social_core.backends.oauth import BaseOAuth2
from django.conf import settings

import logging

logger = logging.getLogger(__name__)


class OpenIdConnect(BaseOAuth2):
    """ OpenID Connect backend """

    name = settings.OPEN_ID_NAME
    EXTRA_DATA = []
    user = None
    ID_KEY = settings.ID_KEY
    ACCESS_TOKEN_METHOD = settings.ACCESS_TOKEN_METHOD
    ID_TOKEN_ISSUER = settings.ID_TOKEN_ISSUER
    OIDC_ENDPOINT = settings.OIDC_ENDPOINT
    AUTHORIZATION_URL = settings.AUTHORIZATION_URL
    ACCESS_TOKEN_URL = settings.ACCESS_TOKEN_URL
    REFRESH_TOKEN_URL = settings.REFRESH_TOKEN_URL
    REDIRECT_STATE = settings.REDIRECT_STATE

    def auth_allowed(self, response, details):
        remote_groups = settings.REMOTE_GROUP_FILTER(response.get(settings.REMOTE_GROUP_KEY, None))
        internal_groups = Group.objects.filter(name__in=remote_groups)

        content_type = ContentType.objects.get(app_label=settings.CONTENT_TYPE_APP_LABEL,
                                               model=settings.CONTENT_TYPE_MODEL)
        login_perm = Permission.objects.get(codename=settings.PERMISSION_MODEL_CODENAME, content_type=content_type)
        for group in internal_groups:
            try:
                group.permissions.get(pk=login_perm.pk)
                logger.info('Login Allowed for group {}'.format(group.name))
                return True
            except ObjectDoesNotExist:
                pass

        logger.info('Login Forbidden. No allowed group for: {}->{}'.format(response.get(settings.ID_KEY),
                                                                           response.get(settings.REMOTE_GROUP_KEY)))
        return False

    def get_user_details(self, response):
        """ Return user details from Open Id account" """
        return settings.GET_USER_DETAILS(response)

    def get_user(self, user_id, **kwargs):
        return get_user_model().objects.get(pk=user_id)

    def user_data(self, access_token, *args, **kwargs):
        """ Load and parse user data from service """
        url = settings.USER_INFO_URL + urlencode({'access_token': access_token})
        result = self.get_json(url)
        return result

    def auth_html(self):
        super(OpenIdConnect, self).auth_html()



"""
Django settings for django_openid_connect project.

Generated by 'django-admin startproject' using Django 1.11.17.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'social_django',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'social_django.middleware.SocialAuthExceptionMiddleware',
]

ROOT_URLCONF = 'django_openid_connect.urls'

REST_FRAMEWORK = {
    'DEFAULT_VERSIONING_CLASS': 'rest_framework.versioning.URLPathVersioning',
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.SessionAuthentication',
    ),
    'TEST_REQUEST_RENDERER_CLASSES': (
        'rest_framework.renderers.MultiPartRenderer',
        'rest_framework.renderers.JSONRenderer',
        'rest_framework.renderers.TemplateHTMLRenderer'
    ),
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    )
}

AUTHENTICATION_BACKENDS = (
    'social_core.backends.open_id.OpenIdAuth',
    'django.contrib.auth.backends.ModelBackend',
    'django_openid_connect.backends.OpenIdConnect'
)


SOCIAL_AUTH_AUTHENTICATION_BACKENDS = (
    'social_core.backends.open_id.OpenIdAuth',
    'django_openid_connect.backends.OpenIdConnect'
)

SOCIAL_AUTH_PERUN_PIPELINE = (
    'social_core.pipeline.user.user_details',
    'social_core.pipeline.social_auth.social_uid',
    'social_core.pipeline.social_auth.social_details',
    'social_core.pipeline.social_auth.load_extra_data',
    'social_core.pipeline.user.create_user',
    'django_openid_connect.pipeline.user_groups',
    'social_core.pipeline.social_auth.auth_allowed',
    'social_core.pipeline.social_auth.associate_user',
    'social_core.pipeline.social_auth.social_user',
)

SOCIAL_AUTH_POSTGRES_JSONFIELD = True
SOCIAL_AUTH_URL_NAMESPACE = 'django_openid_connect'

TEMPLATE_CONTEXT_PROCESSORS = (
    'social_django.context_processors.backends',
    'social_django.context_processors.login_redirect',
)


OPEN_ID_NAME = 'open_id_name'
CONTENT_TYPE_APP_LABEL = 'django_openid_connect'
CONTENT_TYPE_MODEL = 'user'
PERMISSION_MODEL_CODENAME = 'login_user'
USER_INFO_URL = 'https://login.com/oidc/userinfo?'
REMOTE_GROUP_KEY = 'group'
ID_KEY = 'id'
ACCESS_TOKEN_METHOD = 'POST'
ID_TOKEN_ISSUER = 'https://login.com/oidc/'
OIDC_ENDPOINT = 'https://login.com/oidc/'
AUTHORIZATION_URL = 'https://login.com/oidc/authorize'
ACCESS_TOKEN_URL = 'https://login.com/oidc/token'
REFRESH_TOKEN_URL = 'https://login.com/oidc/token'
REDIRECT_STATE = False

GET_USER_DETAILS = lambda response: response

REMOTE_GROUP_FILTER = lambda groups: groups


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_openid_connect']

package_data = \
{'': ['*']}

install_requires = \
['django-rest-framework-social-oauth2==1.1.0', 'urnparse==0.2.0']

setup_kwargs = {
    'name': 'django-openid-connect',
    'version': '1.1.0rc2',
    'description': 'Django library providing OpenID Connect auth backend',
    'long_description': None,
    'author': 'Miroslav Bauer',
    'author_email': 'bauer@cesnet.cz',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
